/*
 * @(#) XATerminatorImpl.java	1.2 02/01/15
 *
 * JOTM: Java Open Transaction Manager 
 *
 * This module was originally developed by 
 *
 *  - Bull S.A. as part of the JOnAS application server code released in 
 *    July 1999 (www.bull.com)
 * 
 * --------------------------------------------------------------------------
 *  The original code and portions created by Bull SA are 
 *  Copyright (c) 1999 BULL SA  
 *  All rights reserved.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * -Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer. 
 *
 * -Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 * -------------------------------------------------------------------------- 
 *
 * Contributor: 
 * 04/01/15 Tony Ortiz
 *
 * --------------------------------------------------------------------------
 * $Id: XATerminatorImpl.java,v 1.1 2004/03/08 21:31:44 tonyortiz Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jotm;

import java.rmi.NoSuchObjectException;
import java.io.Serializable;
import java.util.Collections;
import java.util.EmptyStackException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Stack;
import java.util.Vector;
import java.util.ArrayList;
import javax.transaction.Status;
import javax.transaction.TransactionRolledbackException;
import javax.transaction.xa.Xid;

import javax.resource.spi.XATerminator;

import javax.transaction.xa.XAException;
import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.InvalidTransactionException;
import javax.transaction.NotSupportedException;
import javax.transaction.RollbackException;
import javax.transaction.Status;
import javax.transaction.SystemException;


/**
 * Implementation of the object that represents an inflow transaction.
 *
 */

public class XATerminatorImpl implements XATerminator {

    // Must be init at null, for clients that do not get UserTransaction
    private transient static XATerminatorImpl unique = null;

    // ------------------------------------------------------------------
    // Constructors
    // ------------------------------------------------------------------

    /**
     * @serial
     */
    private TransactionImpl iftx = null;
    /**
     * @serial
     */
    private Xid xid;
    /**
     * @serial
     */
    private Log mylog;

    // XAException values

    private int errorCode;
    private final static int XA_RBBASE = 100;
    private final static int XA_RBROLLBACK = XA_RBBASE;
    private final static int XA_RBCOMMFAIL = XA_RBBASE + 1;
    private final static int XA_RBDEADLOCK = XA_RBBASE + 2;
    private final static int XA_RBINTEGRITY = XA_RBBASE + 3;
    private final static int XA_RBOTHER = XA_RBBASE + 4;
    private final static int XA_RBPROTO = XA_RBBASE + 5;
    private final static int XA_RBRBTIMEOUT = XA_RBBASE + 6;
    private final static int XA_RBTRANSIENT = XA_RBBASE + 7;
    private final static int XA_RBEND = XA_RBTRANSIENT;
    private final static int XA_NOMIGRATE = 9;
    private final static int XA_HEURHAZ = 8;
    private final static int XA_HEURCOM = 7;
    private final static int XA_HEURRB = 6;
    private final static int XA_HEURMIX = 5;
    private final static int XA_NOTUSED1 = 4;
    private final static int XA_RDONLY = 3;
    // Error Codes
    private final static int XAER_RMERR = -3;
    private final static int XAER_NOTA = -4;
    private final static int XAER_INVAL = -5;
    private final static int XAER_PROTO = -6;
    private final static int XAER_RMFAIL = -7;
    private final static int XAER_DUPID = -8;
    private final static int XAER_OUTSIDE = -9;


    // ---------------------------------------------------------------
    // Constructors
    // ---------------------------------------------------------------

    /**
     * Constructor for create
     *
     */

    public XATerminatorImpl() throws XAException {

        if (TraceTm.jotm.isDebugEnabled()) {
            TraceTm.jotm.debug("create XATerminator");
        }

        unique = this;
    }

    // ------------------------------------------------------------------
    // Inflow Transaction implementation
    // ------------------------------------------------------------------

    /**
     * Gets a String that represents the inflow transaction name.
     *
     * @return			Transaction Name
     */

    public String get_transaction_name () throws XAException {

        TraceTm.jotm.debug("XATerminatorImpl.get_transaction_name()");

        return xid.toString();
    }

    /**
     * Commits the global transaction specified by xid.
     *
     * @param xid A global transaction identifier
     * @param onePhase If true, the resource manager should use one-phase commit protocol
     * to commit the work done on behalf of xid.
     * 
     * @exception XAException An error has occurred. Possible XAExceptions
     * are XA_HEURHAZ, XA_HEURCOM, XA_HEURRB, XA_HEURMIX, XAER_RMERR,
     * XAER_RMFAIL, XAER_NOTA, XAER_INVAL, or XAER_PROTO.
     *	
     * If the resouce manager did not commit the transaction and the parameter onePhase is set to
     * true, the resource manager may throw one of the XA_RB* exceptions. Upon return, the
     * resource manager has rolled back the branch's work and has released all help resources.
     */

    public void commit (Xid xid, boolean onePhase) throws XAException {

        if (TraceTm.jotm.isDebugEnabled()) {
            TraceTm.jotm.debug("commit xid="+ xid + "onePhase="+ onePhase);
        }

        // get the transaction associated with the passed Xid
        // if the Xid passed was not corrected the commit will fail

        iftx = Current.getCurrent().getTxByXid(xid);

        try {
            iftx.commit();
        } catch (Exception e) {
            TraceTm.jotm.error("XATerminatorImpl.commit(): commit raised exception ", e);
        }
    }

    /**
     * Informs the resource manager to roll back work done on behalf of a transaction branch.
     *
     * @param xid A global transaction identifier.
     * 
     * @exception XAException An error has occurred. Possible XAExceptions
     * are XA_HEURHAZ, XA_HEURCOM, XA_HEURRB, XA_HEURMIX, XAER_RMERR,
     * XAER_RMFAIL, XAER_NOTA, XAER_INVAL, or XAER_PROTO.
     * 
     * If the transaction branch is already marked rollback-only the resource manager may throw
     * one of the XA_RB* exceptions. Upon return, the resource manager has rolled back the
     * branch's work and has released all held resources.
     */

    public void rollback (Xid xid) throws XAException {

        if (TraceTm.jotm.isDebugEnabled()) {
            TraceTm.jotm.debug("rollback xid="+ xid);
        }

        // get the transaction associated with the passed Xid
        // if the Xid passed was not corrected the rollback will fail

        iftx = Current.getCurrent().getTxByXid(xid);

        try {
            iftx.rollback();
        } catch (Exception e) {
            TraceTm.jotm.error("XATerminatorImpl.rollback(): rollback raised exception ", e);
        }
    }

    /**
     * Ask the resource manager to prepare for a transaction commit of the transaction specified in xid.
     *
     * @param xid A global transaction identifier.
     * 
     * @exception XAException An error has occurred. Possible XAExceptions
     * are XA_HEURHAZ, XA_HEURCOM, XA_HEURRB, XA_HEURMIX, XAER_RMERR,
     * XAER_RMFAIL, XAER_NOTA, XAER_INVAL, or XAER_PROTO.
     *
     * @return A value indicating the resource manager's vote on the outcome of the transaction. The
     * possible values are: XA_RDONLY or XA_OK. These constants are defined in
     * javax.transaction.xa.XAResource interface. If the resource manager wants to roll back
     * the transaction, it should do so by raising an appropriate XAException in the prepare
     * method.
     */

    public int prepare (Xid xid) throws XAException {

        int ret = 0;

        if (TraceTm.jotm.isDebugEnabled()) {
            TraceTm.jotm.debug("prepare xid="+ xid);
        }

        // get the transaction associated with the passed Xid
        // if the Xid passed was not corrected the prepare will fail

        iftx = Current.getCurrent().getTxByXid(xid);

        try {
            ret = iftx.prepare();
        } catch (Exception e) {
            TraceTm.jotm.error("XATerminatorImpl.prepare(): prepare raised exception ", e);
        }

        return ret;
    }

    /**
     * Tells the resource manager to forget about a heuristically completed transaction branch.
     *
     * @param xid A global transaction identifier.
     * 
     * @exception XAException An error has occurred. Possible XAExceptions
     * are XAER_RMERR, XAER_RMFAIL, XAER_NOTA, XAER_INVAL, or XAER_PROTO.
     */

    public void forget (Xid xid) throws XAException {

        TraceTm.jotm.debug("XATerminatorImpl.forget()");

        // get the transaction associated with the passed Xid
        // if the Xid passed was not corrected the forget will fail

        try {
            Current.getCurrent().forgetTx(xid);
        } catch (Exception e) {
            TraceTm.jotm.error("XATerminatorImpl.forget(): forget raised exception ", e);
        }
    }

    /**
     * Obtains a list of prepared transaction branches from a resource manager. The transaction manager
     * calls this method during recovery to obtain the list of transaction branches that are currently in
     * prepared or heuristically completed states.
     *
     * @param flag One of TMSTARTRSCAN, TMENDRSCAN, TMNOFLAGS. TMHOFLAGS must
     * be used when no other flags are set in the parameter. These constants are defined in
     * javax.transaction.xa.XAResource interface.
     * 
     * @exception XAException An error has occurred. Possible values
     * are XAER_RMERR, XAER_RMFAIL, XAER_INVAL, or XAER_PROTO.
     *
     * @return The resource manager returns zero or more XIDs of the transaction branches that are
     * currently in a prepared or heuristically completed state. If an error occurs during the
     * operation, the resource manager should throw the appropriate XAException.
     */

    public Xid[] recover (int flag) throws XAException {

        TraceTm.jotm.debug("XATerminatorImpl.recover()");

        // get a list of transaction in the prepared or heuristically complete states

        return Current.getCurrent().getPreparedHeuristicXid();

    }

}
