/*
 * @(#) JotmTransactionReceiver
 *
 * JOTM: Java Open Transaction Manager
 * 
 * This module was originally developed by 
 *  - INRIA inside the ObjectWeb Consortium(http://www.objectweb.org)
 *    
 * The original code and portions created by INRIA are 
 * Copyright (C) 2002 - INRIA (www.inria.fr)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * -Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer. 
 *
 * -Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * --------------------------------------------------------------------------
 * $Id: JotmTransactionReceiver.java,v 1.1 2004/03/09 19:27:48 tonyortiz Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jotm.jta.jeremie;

import org.objectweb.jotm.Current;
import org.objectweb.jotm.TransactionContext;

/**
 * @author Philippe Durieux
 * 
 * Contributor(s): Guillaume Riviere
 */
public class JotmTransactionReceiver {
   

    public JotmTransactionReceiver() {
    }
  
    /**
     * Receiver is receiving a request with an attached propagation context.
     */
    public void received_request(TransactionContext pctx) {
	// Gets Current object (always existing in EJBServer)
	Current ut = Current.getCurrent();
	if (ut != null) {
	    // Associates the current thread with this propagation context
	    ut.setPropagationContext(pctx, false);
	}
    }
  
    /** 
     * Receiver is sending a reply.
     * Return the propagation context to the Orb.
     */
    public TransactionContext sending_reply() {
	// Gets Current object (always existing in EJBServer)
	TransactionContext pctx = null;
	Current ut = Current.getCurrent();  
	if (ut != null) {
	    pctx = ut.getPropagationContext(false);
	} 
	if (pctx != null) {
	    // Dissociates first the current thread from any propagation context
	    // to make sure the thread will not be kept associated with an old tx.
	    ut.setPropagationContext(null, false);
	} 
	return pctx;
    }
   
}
