/*
 * @(#) TSHandlerFactory
 *
 * JOTM: Java Open Transaction Manager
 * 
 * This module was originally developed by 
 *  - INRIA inside the ObjectWeb Consortium(http://www.objectweb.org)
 *    
 * The original code and portions created by INRIA are 
 * Copyright (C) 2002 - INRIA (www.inria.fr)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * -Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer. 
 *
 * -Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * --------------------------------------------------------------------------
 * $Id: TSHandlerFactory.java,v 1.1 2004/03/09 19:30:15 tonyortiz Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jotm.jta.jeremie;

import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.InternalException;
import org.objectweb.jonathan.apis.kernel.JonathanException;
import org.objectweb.jonathan.apis.presentation.MarshallerFactory;
import org.objectweb.jonathan.libs.kernel.GenericFactory;


/** 
 * {@link TSHandler TSHandler} 
 */
public class TSHandlerFactory extends GenericFactory {

   /** 
    * Name used to designate the context containing the TSHandler components. 
    */
   static public final String ts_context_name = "/jotm/transaction/TSHandler";

   /** 
    * Returns the components required to create a new 
    * {@link TSHandler TSHandler} instance.
    * <p>
    * <code>_c</code> must contain a (non null) component of name 
    * "MarshallerFactory" of type {@link MarshallerFactory MarshallerFactory}.
    * <p>
    * It may contain a component of name "id", of type Integer; this id
    * should then represent the service id to use for the created service.
    * <p>
    * It may contain a component of name "sender", of type Sender,
    * and a component of name "receiver", of type Receiver.
    * 
    * @param c a {@link Context Context} instance;
    * @return the components needed to create an TSHandler instance.
    */
   protected Object[] getUsedComponents(Context c) {
      
      Object[] used_components = { 
	  Context.NO_VALUE, Context.NO_VALUE, Context.NO_VALUE, Context.NO_VALUE 
      };

      if (c != null) {
         used_components[0] = c.getValue("id", (char) 0);
         used_components[1] = c.getValue("sender", (char) 0);
         used_components[2] = c.getValue("receiver", (char) 0);
         used_components[3] = c.getValue("MarshallerFactory", (char) 0);
      }
      if (! (used_components[0] instanceof Integer)) {
         used_components[0] = new Integer(Integer.MAX_VALUE);
      }
      if (! (used_components[1] instanceof JotmTransactionSender)) {
         used_components[1] = null;
      }
      if (! (used_components[2] instanceof JotmTransactionReceiver)) {
         used_components[2] = null;
      }
      if (! (used_components[3] instanceof MarshallerFactory)) {
         throw new InternalException("Component MarshallerFactory of type MarshallerFactory is required by TSHandler.");
      }
      
      return used_components;
   }
   
   /** 
    * Returns a new TSHandler instance created using the provided 
    * components.
    *
    * @param c a context; 
    * @param components components to be used to create a new TSHandler instance;
    * @return a new TSHandler instance;
    * @exception JonathanException if something goes wrong.
    */
   protected Object newInstance(Context c, Object[] components) throws JonathanException {
      return new TSHandler(c, components);
   }
}

