/*
 * The contents of this file are subject to the JOnAS Public License Version
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations
 * under
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999,2001 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): Frederic Maistre, Nicolas Tachker.
 *
 * --------------------------------------------------------------------------
 * $Id: JObjectFactory.java,v 1.2 2002/01/11 08:19:44 jonas Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_jms; 

import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.Name;
import javax.naming.Reference;
import javax.naming.spi.ObjectFactory;
import org.objectweb.jonas_jms.api.JmsManager;

/** 
 * Factory used by JNDI lookup to get Topic or Queue Connection Factories.
 * These factories are managed by the JMS Manager.
 * @see	   javax.naming.spi.ObjectFactory
 */
public class JObjectFactory implements ObjectFactory {

    /** 
     * Always return the unique ConnectionFactory stored in the JmsManager
     */
    public Object getObjectInstance(Object refObj, Name name, Context nameCtx, Hashtable env) throws Exception {

	Reference ref = (Reference) refObj;
	JmsManager jms = JmsManagerImpl.getJmsManager();
	String clname = ref.getClassName();

	TraceJms.debug("JObjectFactory getObjectInstance "+clname);

	if (clname.equals("org.objectweb.jonas_jms.JQueueConnectionFactory")) {
	    // QueueConnectionFactory
	    return jms.getQueueConnectionFactory();
	} else if (clname.equals("org.objectweb.jonas_jms.JTopicConnectionFactory")) {
	    // TopicConnectionFactory
	    return jms.getTopicConnectionFactory();
	} else {
	    TraceJms.error("JObjectFactory - Bad class name: "+clname);
	    return null;
	}
    }
}
