/*
 * The contents of this file are subject to the JOnAS Public License Version
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations
 * under
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999,2001 Bull S.A. All Rights Reserved.
 *
 * Contributor(s):
 *
 * --------------------------------------------------------------------------
 * $Id: JSession.java,v 1.1 2001/12/21 10:53:48 jonas Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_jms;

import java.io.Serializable;
import javax.jms.BytesMessage;
import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import javax.jms.StreamMessage;
import javax.jms.TextMessage;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;
import javax.transaction.xa.XAException;
import javax.transaction.xa.XAResource;
import javax.transaction.xa.Xid;

public abstract class JSession implements Session, XAResource {

    protected XAResource xares = null;	// The underlaying XAResource 
    protected boolean txover = true;
    protected Transaction currtx = null;
    protected boolean closed = false;
    protected JConnection jconn;
    protected static TransactionManager tm = null;

    /**
     * Constructor
     */
    public JSession(JConnection jconn) {
	this.jconn = jconn;
	if (tm == null) {
	    tm = JmsManagerImpl.getTransactionManager();
	}
    }

    // -----------------------------------------------------------------------
    // Internal Methods
    // -----------------------------------------------------------------------

    /**
     * Get the underlaying MOM Session.
     */
    abstract protected Session getMOMSession() throws JMSException;

    abstract protected void MOMSessionClose();

    /**
     * 
     */
    protected void PhysicalClose() {
	MOMSessionClose();
    }

    // -----------------------------------------------------------------------
    // Session Implementation
    // -----------------------------------------------------------------------

    /**
     * 
     */
    public void close() throws JMSException {
	TraceJms.debug("JSession: close");
	closed = true;
	if (txover) {
	    PhysicalClose();
	} else {
	    // delist XAResource now, that will lead to an XA-end call
	    if (currtx == null) {
		TraceJms.error("JSession close: should be in a tx");
	    } else {
		try {
		    currtx.delistResource(this, XAResource.TMSUCCESS);
		} catch (SystemException e) {
		    TraceJms.error("JSession close: cannot delist session:"+e);
		    throw new JMSException(e.toString());
		}
	    }
	}
    }

    /**
     * 
     */
    public void commit() throws JMSException {
	TraceJms.debug("JSession: commit");
	throw new JMSException("JSession: commit Operation Not Allowed");
    }

    /**
     * 
     */
    public BytesMessage createBytesMessage() throws JMSException {
	TraceJms.debug("JSession: createBytesMessage");
	return getMOMSession().createBytesMessage();
    }

    /**
     * 
     */
    public MapMessage createMapMessage() throws JMSException {
	TraceJms.debug("JSession: createMapMessage");
	return getMOMSession().createMapMessage();
    }

    /**
     * 
     */
    public Message createMessage() throws JMSException {
	TraceJms.debug("JSession: createMessage");
	return getMOMSession().createMessage();
    }

    /**
     * 
     */
    public ObjectMessage createObjectMessage() throws JMSException {
	TraceJms.debug("JSession: createObjectMessage");
	return getMOMSession().createObjectMessage();
    }

    /**
     * 
     */
    public ObjectMessage createObjectMessage(Serializable object) throws JMSException {
	TraceJms.debug("JSession: createObjectMessage");
	return getMOMSession().createObjectMessage(object);
    }

    /**
     * 
     */
    public StreamMessage createStreamMessage() throws JMSException {
	TraceJms.debug("JSession: createStreamMessage");
	return getMOMSession().createStreamMessage();
    }

    /**
     * 
     */
    public TextMessage createTextMessage() throws JMSException {
	TraceJms.debug("JSession: createTextMessage");
	return getMOMSession().createTextMessage();
    }

    /**
     * 
     */
    public TextMessage createTextMessage(String text) throws JMSException {
	TraceJms.debug("JSession: createTextMessage");
	return getMOMSession().createTextMessage(text);
    }

    /**
     * 
     */
    public MessageListener getMessageListener() throws JMSException {
	TraceJms.debug("JSession: getMessageListener");
	return getMOMSession().getMessageListener();
    }
    
    /**
     * 
     */
    public boolean getTransacted() throws JMSException {
	TraceJms.debug("JSession: getTransacted");
	return getMOMSession().getTransacted();
    }

    /**
     * 
     */
    public void recover() throws JMSException {
	TraceJms.debug("JSession: recover");
	throw new JMSException("JSession: recover Operation Not Allowed");
    }

    /**
     * 
     */
    public void rollback() throws JMSException {
	TraceJms.debug("JSession: rollback");
	throw new JMSException("JSession: rollback Operation Not Allowed");
    }

    /**
     * 
     */
    public void run() {
	TraceJms.debug("JSession: run");
	try {
	    getMOMSession().run();
	} catch (JMSException e) {
	    TraceJms.error("JSession: run "+e);
	}
    }

    /**
     * 
     */
    public void setMessageListener(MessageListener listener) throws JMSException {
	TraceJms.debug("JSession: setMessageListener");
	getMOMSession().setMessageListener(listener);
    }

    // -----------------------------------------------------------------------
    // XAResource Implementation
    // -----------------------------------------------------------------------

    /**
     *
     */	
    public void commit(Xid xid, boolean onePhase) throws XAException {
	TraceJms.debug("JSession: commit");
	TraceJms.debug("Xid = "+xid);
	xares.commit(xid, onePhase);
	if (txover) {
	    TraceJms.error("JSession: commit tx over ???");
	}
	txover = true;
	if (closed) {
	    PhysicalClose();
	}
    }

    /**
     *
     */	
    public void end(Xid xid, int flags) throws XAException {
	TraceJms.debug("JSession: end");
	TraceJms.debug("Xid = "+xid);
	if (txover) {
	    TraceJms.error("JSession: end tx over ???");
	}
	xares.end(xid, flags);
    }

    /**
     *
     */	
    public void forget(Xid xid) throws XAException {
	TraceJms.debug("JSession: forget");
	TraceJms.debug("Xid = "+xid);
	xares.forget(xid);
    }

    /**
     *
     */	
    public int getTransactionTimeout() throws XAException {
	TraceJms.debug("JSession: getTransactionTimeout");
	return xares.getTransactionTimeout();
    }

    /**
     *
     */	
    public boolean isSameRM(XAResource xar) throws XAException {
	TraceJms.debug("JSession: isSameRM");
	return xares.isSameRM(xar);
    }

    /**
     *
     */	
    public int prepare(Xid xid) throws XAException {
	TraceJms.debug("JSession: prepare");
	TraceJms.debug("Xid = "+xid);
	return xares.prepare(xid);
    }

    /**
     *
     */	
    public Xid[] recover(int flag) throws XAException {
	TraceJms.debug("JSession: recover");
	return xares.recover(flag);
    }

    /**
     *
     */	
    public void rollback(Xid xid) throws XAException {
	TraceJms.debug("JSession: rollback");
	TraceJms.debug("Xid = "+xid);
	xares.rollback(xid);
	if (txover) {
	    TraceJms.error("JSession: rollback tx over ???");
	}
	txover = true;
	if (closed) {
	    PhysicalClose();
	}
    }

    /**
     *
     */	
    public boolean setTransactionTimeout(int seconds) throws XAException {
	TraceJms.debug("JSession: setTransactionTimeout");
	return xares.setTransactionTimeout(seconds);
    }

    /**
     *
     */	
    public void start(Xid xid, int flags) throws XAException {
	TraceJms.debug("JSession: start");
	TraceJms.debug("Xid = "+xid);
	txover = false;
	xares.start(xid, flags);
    }


}

