/*
 * @(#) OTSInterceptor.java	1.0 02/07/15
 *
 * JOTM: Java Open Transaction Manager
 * 
 * This module was originally developed by 
 *  - INRIA inside the ObjectWeb Consortium(http://www.objectweb.org)
 *    
 * The original code and portions created by INRIA are 
 * Copyright (C) 2002 - INRIA (www.inria.fr)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * -Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer. 
 *
 * -Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * --------------------------------------------------------------------------
 * $Id: OTSInterceptor.java,v 1.5 2003/12/05 20:56:46 degolf Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jotm.ots;

import org.omg.CORBA.Any;
import org.omg.CORBA.LocalObject;
import org.omg.CosTransactions.PropagationContextHelper;
import org.omg.DynamicAny.DynAnyFactory;
import org.omg.DynamicAny.DynAnyFactoryHelper;
import org.omg.IOP.Codec;
import org.omg.PortableInterceptor.ORBInitInfo;
import org.omg.PortableInterceptor.ORBInitInfoPackage.InvalidName;



public abstract class OTSInterceptor extends LocalObject {

    // Used to encode/decode the call
    protected Codec codec;


    // for identifying client or server.
    protected DynAnyFactory dynAnyFactoryS_;


    
    // Service context identifier
    protected final int TX_CTX_ID = 100;


    /**
     * constructor
     */

    public OTSInterceptor(ORBInitInfo info) {

	
	// Get the codec factory
	org.omg.IOP.CodecFactory factory = info.codec_factory();
	if(factory == null) throw new RuntimeException();

	// Create codec
	org.omg.IOP.Encoding how = new org.omg.IOP.Encoding();
	how.major_version = 1;
	how.minor_version = 0;
	how.format = org.omg.IOP.ENCODING_CDR_ENCAPS.value;

	try {
	    codec = factory.create_codec(how);
	} catch(org.omg.IOP.CodecFactoryPackage.UnknownEncoding ex) {
	    throw new RuntimeException();
	}

	if(codec == null) throw new RuntimeException();

	//
	// Get the dynamic any factory
	//
	DynAnyFactory dynAnyFactory = null;
	try {
	    org.omg.CORBA.Object obj = info.resolve_initial_references("DynAnyFactory");
	    dynAnyFactory = DynAnyFactoryHelper.narrow(obj);
	} catch(InvalidName ex) {
	    throw new RuntimeException();
	} catch(org.omg.CORBA.BAD_PARAM ex) {
	    throw new RuntimeException();
	}

	dynAnyFactoryS_ = dynAnyFactory;
    } // end constructor



    protected Any create_any() 
	throws org.omg.DynamicAny.DynAnyFactoryPackage.InconsistentTypeCode{
	org.omg.DynamicAny.DynAny dynAny = dynAnyFactoryS_.create_dyn_any_from_type_code(PropagationContextHelper.type());
	return dynAny.to_any();
		    
    }

}

